package com.muyu.data.processing.controller;


import com.alibaba.fastjson.JSONObject;
import com.muyu.common.kafka.constants.KafkaConstants;
import com.muyu.data.processing.domain.IotDbData;
import com.muyu.data.processing.domain.req.TestReq;
import com.muyu.data.processing.domain.resp.TestResp;
import com.muyu.data.processing.strategy.root.RootStrategy;
import jakarta.annotation.Resource;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import lombok.extern.slf4j.Slf4j;

/**
 * 测试控制层
 * @Author: 胡杨
 * @Name: Test
 * @Description:
 * @CreatedDate: 2024/9/27 上午10:54
 * @FilePath: com.muyu.data.processing.controller
 */
@Slf4j
@RestController
@RequestMapping("/Test")
public class TestController {
    @Resource
    private KafkaProducer<String,String> kafkaProducer;

    @GetMapping("/testKafka")
    public void sendMsg(@RequestParam("msg") String msg) {
        try {
            IotDbData iotDbData = IotDbData.builder()
                    .timestamp(System.currentTimeMillis())
                    .vin("vin666")
                    .key("test")
                    .label("测试数据")
                    .value("Kafka测试")
                    .type("String")
                    .build();
            String jsonString = JSONObject.toJSONString(iotDbData);
            ProducerRecord<String, String> producerRecord = new ProducerRecord<>(KafkaConstants.KafkaTopic, jsonString);
            kafkaProducer.send(producerRecord);
            System.out.println("同步消息发送成功: " + msg);
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("同步消息发送失败: " + msg);
        }
    }


    @Resource
    private RootStrategy rootStrategy;

    @PostMapping("/testStrategy")
    public TestResp testStrategy(@RequestBody TestReq testReq) {
        return rootStrategy.applyStrategy(testReq);
    }
}
