package com.muyu.data.processing.kafka;


import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.nacos.shaded.com.google.common.collect.Lists;
import com.muyu.common.kafka.constants.KafkaConstants;
import com.muyu.data.processing.domain.IotDbData;
import com.muyu.data.processing.domain.KafkaData;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.Collection;
import java.util.List;

/**
 * @Author: 胡杨
 * @Name: KafkaConsumerService
 * @Description: kafka消费者
 * @CreatedDate: 2024/9/27 上午9:27
 * @FilePath: com.muyu.data.processing.kafka
 */

@Slf4j
@Component
public class KafkaConsumerService implements InitializingBean {
    @Resource
    private KafkaConsumer kafkaConsumer;

    @Override
    public void afterPropertiesSet() throws Exception {
        Thread thread = new Thread(() -> {
            log.info("启动线程监听Topic: {}", KafkaConstants.KafkaTopic);
            ThreadUtil.sleep(1000);
            Collection<String> topics = Lists.newArrayList(KafkaConstants.KafkaTopic);
            kafkaConsumer.subscribe(topics);
            while (true) {
                ConsumerRecords<String, String> consumerRecords = kafkaConsumer.poll(Duration.ofMillis(1000));
                for (ConsumerRecord consumerRecord : consumerRecords) {
                    //1.从ConsumerRecord中获取消费数据
                    String originalMsg = (String) consumerRecord.value();
                    log.info("从Kafka中消费的原始数据: " + originalMsg);
                    //2.把消费数据转换为DTO对象
                    List<KafkaData> kafkaDataList = JSONUtil.toList(originalMsg, KafkaData.class);
                    log.info("消费数据转换为DTO对象: " + kafkaDataList.toString());
                }
            }
        });
        thread.start();
    }
}
